import * as Electron from 'electron';

import { ObjectUtil } from '@ucap/core';

import { EventHandlerRegistry } from './registries/event-handler.registry';
import { EventAttachType } from './types/event-attach.type';
import { EventHandlerMetadata } from './models/event-handler.metadata';

import { ElectronBrowserWindow } from './electron-browser-window';

export abstract class Attacher {
  static attach<T extends ElectronBrowserWindow>(instance: T) {
    const metadatas = EventHandlerRegistry.getEventHandlers(
      ObjectUtil.classOf(instance)
    );

    if (!!metadatas) {
      metadatas.reverse().forEach((metadata) => {
        metadata.eventHandlers.forEach((channels, attachType) => {
          switch (attachType) {
            case EventAttachType.on:
              {
                channels.forEach((channel) => {
                  instance.native.on(channel as any, (...args: any[]) => {
                    Attacher.invoke(metadata, instance, args);
                  });
                });
              }
              break;
          }
        });
      });
    }
  }

  static dettach<T extends ElectronBrowserWindow>(instance: T) {
    instance.native.removeAllListeners();
  }

  static invoke(
    metadata: EventHandlerMetadata,
    instance: any,
    args: any[]
  ): any {
    if (!!metadata.parent) {
      return Attacher.invoke(metadata.parent, instance, args);
    }

    // tslint:disable-next-line: ban-types
    return (metadata.target.prototype[metadata.propertyKey] as Function).call(
      instance,
      ...args
    );
  }
}
