import { ObjectUtil } from '@ucap/core';

export enum LifecycleHooks {
  AfterInit,
  BeforeDestroy
}

export interface AfterInit {
  ucapAfterInit(): void | Promise<void>;
}

export interface BeforeDestroy {
  ucapBeforeDestroy(): void | Promise<void>;
}

export const hookNameOf = (lifecycleHook: LifecycleHooks) => {
  switch (lifecycleHook) {
    case LifecycleHooks.AfterInit:
      return 'ucapAfterInit';
    case LifecycleHooks.BeforeDestroy:
      return 'ucapBeforeDestroy';
  }

  return undefined;
};

export const getHook = (instance: any, lifecycleHook: LifecycleHooks) => {
  const n = hookNameOf(lifecycleHook);
  if (!n) {
    return undefined;
  }

  const h = instance[n];
  if (!ObjectUtil.isFunction(h)) {
    return undefined;
  }

  return h;
};
