import * as path from 'path';

export const NODE_ENV = 'NODE_ENV';

export class PlatformUtil {
  static isRenderer() {
    return !!process && !!process.type && 'renderer' === process.type;
  }

  static isProduction() {
    return (
      !!process &&
      !!process.env &&
      !!process.env[NODE_ENV] &&
      'production' === process.env[NODE_ENV]
    );
  }

  static isPackaged() {
    if (
      !!process.mainModule &&
      -1 !== process.mainModule.filename.indexOf('app.asar')
    ) {
      return true;
    } else if (
      0 < process.argv.filter((a) => -1 !== a.indexOf('app.asar')).length
    ) {
      return true;
    }

    return false;
  }

  static isWindows() {
    return !!process && !!process.platform && 'win32' === process.platform;
  }
  static isDarwin() {
    return !!process && !!process.platform && 'darwin' === process.platform;
  }
  static isLinux() {
    return !!process && !!process.platform && 'linux' === process.platform;
  }

  static rootPath(basePath: string = './') {
    if (PlatformUtil.isPackaged()) {
      if (PlatformUtil.isWindows()) {
        return path.join(basePath, '..');
      } else {
        return path.join(basePath, '..');
      }
    } else if (PlatformUtil.isProduction()) {
      if (PlatformUtil.isRenderer()) {
        return path.join(basePath, '..');
      } else if (!module.parent) {
        return path.join(basePath, '..');
      } else {
        return path.join(basePath, '..');
      }
    } else if (PlatformUtil.isRenderer()) {
      return path.join(basePath, '..');
    } else {
      return path.join(basePath, '..');
    }
  }
}
