import { LogAppender, LogEntry, LogLevel } from '@ucap/logger';

import electron_log from 'electron-log';

export class ElectronLogAppender extends LogAppender {
  constructor() {
    super();
  }

  log(entry: LogEntry): Promise<boolean> {
    return new Promise<boolean>((resolve, reject) => {
      try {
        switch (entry.level) {
          case LogLevel.Trace:
            electron_log.silly(...entry.args);
            break;
          case LogLevel.Debug:
            electron_log.debug(...entry.args);
            break;
          case LogLevel.Info:
            electron_log.verbose(...entry.args);
            break;
          case LogLevel.Warn:
            electron_log.info(...entry.args);
            break;
          case LogLevel.Error:
            electron_log.warn(...entry.args);
            break;
          case LogLevel.Fatal:
            electron_log.error(...entry.args);
            break;
        }
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  clear(): Promise<boolean> {
    return new Promise<boolean>((resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }
}
